@@ -4,18 +4,20 @@ import os
 import pathlib
 import re
 import typing
+import urllib.parse
 
 from . import commands
 from . import githost
 
 
 TODAY = datetime.date.today()
+SSH_GIT_URL = re.compile('(?P<user>.+)@(?P<host>.+):(?P<path>.+)')
+DEFAULT_API_URL = "https://api.github.com"
 
 
 @dataclasses.dataclass
 class GitHostApiAccessInfo:
-    platform: str = "github"
-    api_base_url: str = "https://api.github.com"
+    api_base_url: str = DEFAULT_API_URL
     auth_token_file: str = None
     auth_token_env_var: str = None
 
@@ -24,7 +26,7 @@ class GitHostApiAccessInfo:
 
     def get_auth_token(self):
         if self.auth_token_file:
-            return pathlib.Path(self.auth_token_file).expanduser().read_text().strip()
+            return pathlib.Path(self.auth_token_file).expanduser().read_text(encoding='utf-8').strip()
         if self.auth_token_env_var:
             return os.environ[self.auth_token_env_var]
         raise ValueError(
@@ -34,6 +36,7 @@ class GitHostApiAccessInfo:
 
 @dataclasses.dataclass
 class Config:
+    platform: str = None
     path: str = "."
     max_age: int = 90
 
@@ -43,20 +46,37 @@ class Config:
     calm_branches: typing.Sequence = ("gh-pages", "master", "prod", "maint(enance)?/.*")
     ignore_branches_without_pull_request: bool = False
 
-    host_owner: str = None
-    host_url: str = "https://github.com/{owner}/{repo}/tree/{branch}"
     host_api_access: dict = None
 
+    # Cannot be configured, but is automatically filled in instead. Default value is irrelevant and only here to
+    # initialize the field.
+    host: str = dataclasses.field(default=None, init=False)
+    host_owner: str = dataclasses.field(default=None, init=False)
+    repo_name: str = dataclasses.field(default=None, init=False)
+
     def __post_init__(self):
         if self.host_api_access:
             self.host_api_access = GitHostApiAccessInfo(**self.host_api_access)
 
+        self.host, self.host_owner, self.repo_name = get_repository_info(self.path)
+
+        if not self.platform:
+            if "gitlab" in self.host.lower():
+                self.platform = "gitlab"
+                if self.host_api_access and self.host_api_access.api_base_url == DEFAULT_API_URL:
+                    self.host_api_access.api_base_url = f"https://{self.host}/api/v4"
+            else:
+                self.platform = "github"  # non-breaking change: keep the previous default value
+
     def ignore_branch(self, branch):
         for calm_branch_regexp in self.calm_branches:
             if re.match(calm_branch_regexp, branch):
                 return True
         return False
 
+    def get_branch_url(self, branch):
+        return f"https://{self.host}/{self.host_owner}/{self.repo_name}/tree/{branch}"
+
 
 @dataclasses.dataclass
 class BranchInfo:
@@ -77,21 +97,30 @@ class BranchInfo:
         return details
 
 
-def get_repository_name(path):
-    """Extract the repository name from the origin remote."""
-    # When we can use git >= 2.7, we'll be able to use `git remote get-url origin`.
-    url = commands.get_output(("git", "config", "--get", "remote.origin.url"), cwd=path)[0]
+def get_repository_info(path):
+    """Extract the repository owner and name from the origin remote."""
+    # Compared to "git config --get remote.origin.url" or "git remote get-url origin", this handles aliases defined in
+    # git config.
+    remote_url = commands.get_output(("git", "ls-remote", "--get-url"), cwd=path)[0]
     # It usually looks like "git@github.com:Polyconseil/check-oldies.git"
     # but could also look like "poly:check-fixmes.git" if you have an
     # "[url]" section with "insteadOf" in your Git configuration.
-    if url.endswith(".git"):
-        url = url[:-4]
-    return url.rsplit("/", 1)[-1].rsplit(":", 1)[-1]
+    if remote_url.endswith(".git"):
+        remote_url = remote_url[:-4]
+    parsed = urllib.parse.urlparse(remote_url)
+    if parsed.scheme in ('http', 'https'):
+        path = parsed.path.lstrip('/')
+        host_owner, repo_name = path.split('/', 1)
+        return parsed.netloc, host_owner, repo_name
+    ssh_match = SSH_GIT_URL.match(remote_url)
+    if ssh_match:
+        path = ssh_match.group('path')
+        host_owner, repo_name = path.split('/', 1)
+        return ssh_match.group('host'), host_owner, repo_name
+    raise ValueError(f"Could not parse remote origin and determine the Git host: '{remote_url}'")
 
 
 def get_branches(config: Config):
-    repo = get_repository_name(config.path)
-
     branches = []
     for branch in commands.get_output(("git", "branch", "--remotes"), cwd=config.path):
         branch = branch.strip()
@@ -108,12 +137,11 @@ def get_branches(config: Config):
         email, date, *_rest = output.split(" ")
         date = datetime.date(*[int(s) for s in date.split("-")])
         age = (TODAY - date).days
-        url = config.host_url.format(owner=config.host_owner, repo=repo, branch=branch)
         branches.append(
             BranchInfo(
-                repo=repo,
+                repo=config.repo_name,
                 name=branch,
-                url=url,
+                url=config.get_branch_url(branch=branch),
                 author=email,
                 age=age,
                 is_old=age > config.max_age,
@@ -124,9 +152,9 @@ def get_branches(config: Config):
         return ()
 
     if config.host_api_access:
-        pr_getter = githost.PullRequestGetter(config.host_owner, config.host_api_access)
+        pr_getter = githost.PullRequestGetter(config.platform, config.host_owner, config.host_api_access)
         for branch in branches:
-            branch.pull_request = pr_getter.get_pull_request(repo, branch.name)
+            branch.pull_request = pr_getter.get_pull_request(config.repo_name, branch.name)
 
     return [
         branch
